import {Component, OnInit} from '@angular/core';
import {ActivatedRoute, Router} from '@angular/router';
import {SpeciesAnalysis} from 'src/app/models/SpeciesAnalysis';
import {DataService} from 'src/app/services/data.service';
import {SLFDatabase} from "../../models/SLFDatabase";
import {ModelAnalysis} from "../../models/ModelAnalysis";
import {RegionModelAnalysis} from "../../models/RegionModelAnalysis";
import {RegionAnalysis} from "../../models/RegionAnalysis";
import {RegionTreeDistance} from "../../models/RegionTreeDistance";
import { NamedFamilyAnalysis } from 'src/app/models/NamedFamilyAnalysis';

@Component({
  selector: 'slf-result',
  templateUrl: './result.component.html',
  styleUrls: ['./result.component.scss']
})
export class ResultComponent implements OnInit {
  public id: string;
  public readonly family: string;
  private SLFDatabase?: SLFDatabase;
  private speciesAnalysis?: SpeciesAnalysis;
  public speciesAnalysisNames: string[];

  public readonly displayedColumns: string[] =
    ['predicted', 'region', 'model', 'min_tree_distance', 'isoelectric_point'];

  constructor(
    private readonly route: ActivatedRoute,
    private readonly router: Router,
    private readonly dataService: DataService
  ) {
    const paramMap = this.route.snapshot.paramMap;
    if (!(paramMap.has('family') && paramMap.has('id'))) {
      this.router.navigate(['/']);
      throw new Error('Missing family and id');
    }
    this.family = <string>paramMap.get('family');
    this.id = <string>paramMap.get('id');
    this.speciesAnalysisNames = [];
  }

  public ngOnInit(): void {
    console.log(this.id)
    this.dataService.getMultipleFamilyAnalysis()
      .subscribe(slfd => {
        this.SLFDatabase = slfd;
        this.speciesAnalysis = this.SLFDatabase.families[this.family]
          .results[`../s-locus-database/analyses/${this.family}/${this.id}`];
          console.log(this.SLFDatabase.families[this.family])
          this.setSpeciesList(this.SLFDatabase.families[this.family])
      });
  }

  public listModelAnalysis(): ModelAnalysis[] {
    if (this.speciesAnalysis === undefined) {
      return []
    }

    return this.speciesAnalysis.regionAnalysis
      .map(ra => ra.modelAnalysis)
      .reduce((prev, curr) => prev.concat(curr));
  }

  public getModelRegion(analysis: ModelAnalysis): string {
    if (this.speciesAnalysis === undefined) {
      return '';
    }

    for (const regionAnalysis of this.speciesAnalysis.regionAnalysis) {
      if (regionAnalysis.modelAnalysis.includes(analysis)) {
        return regionAnalysis.regionName;
      }
    }

    return '';
  }

  public doesModelHavePredicted(analysis: ModelAnalysis): boolean {
    return analysis.predicted !== undefined;
  }

  public doesModelHavePutative(analysis: ModelAnalysis): boolean {
    return analysis.putative !== undefined;
  }

  public doesModelHaveBoth(analysis: ModelAnalysis): boolean {
    return this.doesModelHavePredicted(analysis) && this.doesModelHavePutative(analysis);
  }

  private getRegionModelAnalysis(analysis: ModelAnalysis): RegionModelAnalysis | undefined {
    let model: RegionModelAnalysis | undefined = undefined;

    if (analysis.predicted !== undefined) {
      model = analysis.predicted;
    } else if (analysis.putative !== undefined) {
      model = analysis.putative;
    }

    return model;
  }

  public getMinTreeDistance(analysis: ModelAnalysis): RegionTreeDistance[] {
    const model = this.getRegionModelAnalysis(analysis);

    if (model !== undefined && model.minTreeDistances !== undefined) {
      return model.minTreeDistances;
    } else {
      return [];
    }
  }

  public getIsoelectricPoints(analysis: ModelAnalysis): number[] {
    const model = this.getRegionModelAnalysis(analysis);

    if (model !== undefined && model.isoelectricPoints !== undefined) {
      return model.isoelectricPoints;
    } else {
      return [];
    }
  }

  private setSpeciesList(family_results: NamedFamilyAnalysis){
    let ini=this.id.indexOf("_"+this.family);
    let actualSpecie=this.id.substring(0,ini);
    Object.keys(family_results['results']).forEach((key: string) => {
    if((key.includes(this.family))&&(key.includes(actualSpecie))){
      this.speciesAnalysisNames.push(family_results['results'][key].speciesName);
    }
  })

  }

  public onChangeResults(value: string): void {
    this.router.navigate(['/species/'+this.family+'/'+value])
  .then(() => {
    window.location.reload();
  });
  }
}
